<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" version="1.0" encoding="iso-8859-1" indent="yes"/>
<xsl:strip-space elements="volist vo"/>

<xsl:variable name="lcletters">abcdefghijklmnopqrstuvwxyz</xsl:variable>
<xsl:variable name="ucletters">ABCDEFGHIJKLMNOPQRSTUVWXYZ</xsl:variable>

<!-- Input Parameters -->
<xsl:param name="LocalDefs" select ="''"/>
<xsl:param name="OldDefs" select = "''"/>
<xsl:param name="AddVO" select = "''"/>
<xsl:param name="Debug" select="'False'"/>

<xsl:variable name="root-node" select="/"/>

<xsl:variable name="has-local-defs">
 <xsl:choose>
 <xsl:when test="string($LocalDefs)">
  <xsl:variable name="localservices" select="document($LocalDefs)"/>
  <xsl:choose>
   <xsl:when test="count($localservices/services) &gt; 0">1</xsl:when>
   <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
  </xsl:when>
  <xsl:otherwise>0</xsl:otherwise>
 </xsl:choose> 
</xsl:variable>

<xsl:variable name="has-old-defs">
 <xsl:choose>
 <xsl:when test="string($OldDefs)">
  <xsl:variable name="oldservices" select="document($OldDefs)"/>
  <xsl:choose>
   <xsl:when test="count($oldservices/services) &gt; 0">1</xsl:when>
   <xsl:otherwise>0</xsl:otherwise>
  </xsl:choose>
  </xsl:when>
  <xsl:otherwise>0</xsl:otherwise>
 </xsl:choose> 
</xsl:variable>

<!-- Define Index -->
<xsl:key name="servicename" match="service" use="@name" />

<!-- CopyNode template -->
<xsl:template name="CopyNode">
 <xsl:if test="$Debug = 'False'">
  <xsl:copy>
   <xsl:copy-of select="@*"/>
   <xsl:apply-templates/>
  </xsl:copy>
 </xsl:if>
</xsl:template>

<!-- LogDebug template -->
<xsl:template name="Log">
 <xsl:param name="msg"/>
 <xsl:if test="$Debug = 'True'">
  <xsl:message>DEBUG: <xsl:value-of select="$msg"/></xsl:message>
 </xsl:if>
</xsl:template>

<!-- CopyLocal template -->
<xsl:template name="CopyLocals">
 <xsl:if test="$has-local-defs &gt; 0">
 <xsl:variable name="localservices" select="document($LocalDefs)"/>
  <xsl:text>&#xA;</xsl:text>
  <xsl:comment> Local Definitions </xsl:comment>
  <xsl:text>&#xA;</xsl:text>
  <xsl:for-each select="$localservices/services">
   <xsl:apply-templates/>
  </xsl:for-each>
  <xsl:text>&#xA;</xsl:text>
  <xsl:comment> End Local Definitions </xsl:comment>
  <xsl:text>&#xA;</xsl:text>
 </xsl:if>
</xsl:template>

<!-- KeepOld template -->
<xsl:template name="KeepOld">
 <xsl:if test="$has-old-defs &gt; 0">
  <xsl:variable name="oldservices" select="document($OldDefs)"/>
  <xsl:text>&#xA;</xsl:text>
  <xsl:comment> Old Definitions (No longer available in the Information System) </xsl:comment>
  <xsl:text>&#xA;</xsl:text>
  <xsl:for-each select="$oldservices/services">
   <xsl:apply-templates/>
  </xsl:for-each>
  <xsl:text>&#xA;</xsl:text>
  <xsl:comment> End Old Definitions </xsl:comment>
  <xsl:text>&#xA;</xsl:text>
 </xsl:if>
</xsl:template>

<!-- Default template -->
<xsl:template match="*" priority="0">
 <xsl:call-template name="CopyNode"/>
</xsl:template>

<!-- template for type:myproxy or type:srm -->
<xsl:template match="type" priority="1">
 <xsl:choose>
  <xsl:when test="string(.) = 'myproxy'">
   <xsl:copy>MyProxy</xsl:copy>
  </xsl:when>
  <xsl:when test="string(.) = 'srm'">
   <xsl:copy>SRM</xsl:copy>
  </xsl:when>
  <xsl:otherwise>
   <xsl:call-template name="CopyNode"/>
  </xsl:otherwise>
 </xsl:choose>
</xsl:template>

<!-- Default template -->
<!--<xsl:template match="vo" priority="1">
 <xsl:choose>
  <xsl:when test="string($AddVO)">
    <xsl:if test="not(text() = $AddVO and (string(../../type) = 'SRM' or string(../../type) = 'srm'))">
     <xsl:text>        </xsl:text>
     <xsl:call-template name="CopyNode"/>
     <xsl:text>&#xA;</xsl:text>
    </xsl:if>
  </xsl:when>
  <xsl:otherwise>
     <xsl:text>        </xsl:text>
     <xsl:call-template name="CopyNode"/>
     <xsl:text>&#xA;</xsl:text>
  </xsl:otherwise>
 </xsl:choose>
</xsl:template>
-->                                                                                                                                                         
<!-- template for volist -->
<xsl:template match="volist" priority="1">
 <xsl:copy>
  <xsl:text>&#xA;</xsl:text>
  <xsl:for-each select="vo">
   <xsl:sort select="text()" data-type="text"/>
   <xsl:if test="not(node()) or not(preceding-sibling::node()[.=string(current())])">
    <xsl:if test="not(text() = $AddVO and string(../../type) = 'SRM')">
     <xsl:text>        </xsl:text>
     <xsl:call-template name="CopyNode"/>
     <xsl:text>&#xA;</xsl:text>
    </xsl:if>
   </xsl:if>
  </xsl:for-each>
  <xsl:if test="string($AddVO) and string(../type) = 'SRM'">
     <xsl:text>        </xsl:text>
     <xsl:element name="vo"><xsl:value-of select="$AddVO"/></xsl:element>
     <xsl:text>&#xA;</xsl:text>
  </xsl:if>
  <xsl:text>      </xsl:text>
 </xsl:copy>
</xsl:template>
  
  <!-- template for version -->
  <xsl:template match="service/parameters/version" priority="1">
    <xsl:choose>
      <xsl:when test="string(.) = '1'">
        <xsl:copy>1.1.0</xsl:copy>
      </xsl:when>
      <xsl:when test="string(.) = '1.1.'">
        <xsl:copy>1.1.0</xsl:copy>
      </xsl:when>
      <xsl:when test="string(.) = 'N/A'">
        <xsl:copy>1.1.0</xsl:copy>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="CopyNode"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

<!-- template for site -->
<xsl:template match="site" priority="1">
   <xsl:copy><xsl:value-of select="translate(string(.),$lcletters,$ucletters)"/></xsl:copy>
</xsl:template>

<!-- template for endpoint -->
<xsl:template match="endpoint" priority="1">
 <xsl:copy>
  <!-- Check whether is an URL or a network endpoint -->
  <xsl:choose>
   <xsl:when test="contains(string(.),'://')">
    <!-- It's an Url: tokenize the string -->
    <xsl:variable name="schema" select="substring-before(string(.),'://')"/>
    <xsl:variable name="schemapart" select="substring-after(string(.),'://')"/>
    <xsl:variable name="hostname">
     <xsl:choose>
      <xsl:when test="contains($schemapart,'/')">
       <xsl:value-of select="substring-before($schemapart,'/')"/>
      </xsl:when>
      <xsl:otherwise>
       <xsl:value-of select="$schemapart"/>
      </xsl:otherwise>
     </xsl:choose>
    </xsl:variable>
    <xsl:variable name="urlpart" select="substring-after($schemapart,'/')"/>
    <xsl:value-of select="$schema"/>://<xsl:value-of select="translate($hostname,$ucletters,$lcletters)"/>/<xsl:value-of select="$urlpart"/>
   </xsl:when>
   <xsl:otherwise>
    <!-- It's an network endpoint: translate it as lowercase -->
    <xsl:value-of select="translate(string(.),$ucletters,$lcletters)"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:copy>
</xsl:template>

<!-- template to check if the current element is in one of the additional [LocalDefs,OldDefs] documents -->
<xsl:template name="WhichDocument">
 <xsl:variable name="parent-id" select="generate-id(..)"/>
 <!-- Get ID of LocalDefs/services node -->
 <xsl:variable name="localservices-id">
  <xsl:choose>
   <xsl:when test="$has-local-defs &gt; 0">
    <xsl:variable name="localservices" select="document($LocalDefs)"/>
    <xsl:value-of select="generate-id($localservices/services)"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="''"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:variable>
 <!-- Get ID of OldDefs/services node -->
 <xsl:variable name="oldservices-id">
  <xsl:choose>
   <xsl:when test="$has-old-defs &gt; 0">
    <xsl:variable name="oldservices" select="document($OldDefs)"/>
    <xsl:value-of select="generate-id($oldservices/services)"/>
   </xsl:when>
   <xsl:otherwise>
    <xsl:value-of select="''"/>
   </xsl:otherwise>
  </xsl:choose>
 </xsl:variable>
 <!-- Choose which Document the node belongs to -->
 <xsl:choose>
  <xsl:when test="$localservices-id = $parent-id">local</xsl:when>
  <xsl:when test="$oldservices-id = $parent-id">old</xsl:when>
  <xsl:otherwise>current</xsl:otherwise>
 </xsl:choose>
</xsl:template>

<!-- template for service -->
<xsl:template match="service" priority="1">
 <xsl:variable name="doc-defs"><xsl:call-template name="WhichDocument"/></xsl:variable>
 <xsl:call-template name="Log"><xsl:with-param name="msg">
 Current Service (<xsl:value-of select="@name" />) belongs to <xsl:value-of select="$doc-defs" /> Document
 </xsl:with-param></xsl:call-template>
 <xsl:choose>
  <xsl:when test="generate-id() = generate-id(key('servicename', @name)[1])">
  <!-- Check if a copy exists in the local definitions -->
   <xsl:choose>
    <xsl:when test="($doc-defs = 'current') and 
                    (($has-local-defs &gt; 0) and document($LocalDefs)/services/service[@name = current()/@name])">
    <!-- Service is already there: skip it-->
    <xsl:call-template name="Log"><xsl:with-param name="msg">
    ==> Service (<xsl:value-of select="@name" />) already exists in LocalDefs document
    </xsl:with-param></xsl:call-template>
    </xsl:when>
    <xsl:when test="($doc-defs = 'old') and 
                    (  (($has-local-defs &gt; 0) and document($LocalDefs)/services/service[@name = current()/@name])
                    or $root-node/services/service[@name = current()/@name])">
    <!-- Service is already there: skip it-->
    <xsl:call-template name="Log"><xsl:with-param name="msg">
    ==> Service (<xsl:value-of select="@name" />) already exists in Local or Current Defs documents
    </xsl:with-param></xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
     <!-- Service is Unique: copy it -->
     <xsl:call-template name="CopyNode"/>
    </xsl:otherwise>
   </xsl:choose>
  </xsl:when>
  <xsl:otherwise>
  <!-- Service is already there: skip it-->
  <xsl:call-template name="Log"><xsl:with-param name="msg">
   ==> Service (<xsl:value-of select="@name" />) already exists in this document
  </xsl:with-param></xsl:call-template>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>

<!-- Root template -->
<xsl:template match="/services">
 <xsl:text>&#xA;</xsl:text>
 <xsl:comment>
 services.xml file fixed by fix-sd.xsl
 </xsl:comment>
 <xsl:text>&#xA;</xsl:text>
 <xsl:copy>
  <!-- First Copy the Local Definition: Higher priority-->
  <xsl:call-template name="CopyLocals"/>
  <!-- Copy the newer Definitions -->
  <xsl:apply-templates/>
  <!-- Copy the Old Definitions (currently not available in BDII) : Lower priority -->
  <xsl:call-template name="KeepOld"/>
 </xsl:copy>
</xsl:template>

</xsl:stylesheet>

